@@ -3,7 +3,11 @@ import base
 import logging
 import cubes.model
 import collections
-from collections import OrderedDict
+try:
+    from collections import OrderedDict
+except ImportError:
+    from ordereddict import OrderedDict
+    
 from cubes.common import logger_name
 from collections import deque
 
@@ -27,7 +31,7 @@ except:
 # * [DONE] dimension values pagination
 # * remainder
 # * ratio - aggregate sum(current)/sum(total) 
-
+# * derived measures (should be in builder)
 
 class FactsIterator(object):
     """
@@ -60,7 +64,8 @@ class FactsIterator(object):
 class SQLBrowser(cubes.browser.AggregationBrowser):
     """Browser for aggregated cube computed by :class:`cubes.build.MongoSimpleCubeBuilder` """
     
-    def __init__(self, cube, connection = None, view_name = None, schema = None, view = None, locale = None):
+    def __init__(self, cube, connection=None, view_name=None, schema=None, 
+                    view=None, locale=None):
         """Create a browser.
         
         :Attributes:
@@ -76,9 +81,12 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
         """
         super(SQLBrowser, self).__init__(cube)
 
+        if not cube:
+            raise Exception("Cube is not provided (should be not None)")
+
         self.cube = cube
 
-        if not connection and not view:
+        if (connection is None) and (view is None):
             raise Exception("SQLBrowser requires either connection or view to be provided.")
 
         if locale:
@@ -90,16 +98,20 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
         if not self.fact_key:
             self.fact_key = base.DEFAULT_KEY_FIELD
 
-        if connection:
+        if connection is not None:
             # FIXME: This reflection is somehow slow (is there anotherway how to do it?)
             self.connection = connection
             self.view_name = view_name
+
+            if not self.view_name:
+                raise Exception("No view name specified for browser")
+                
             self.engine = self.connection.engine
             metadata = sqlalchemy.MetaData(bind = self.engine)
 
-            self.view = sqlalchemy.Table(self.view_name, metadata, autoload = True, schema = schema)
+            self.view = sqlalchemy.Table(self.view_name, metadata, autoload=True, schema=schema)
             self.key_column = self.view.c[self.fact_key]
-        elif view:
+        elif view is not None:
             self.connection = view.bind
             self.engine = self.connection.engine
             self.view = view
@@ -107,19 +119,20 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
 
         self.logger = logging.getLogger(logger_name)
 
-    def aggregate(self, cuboid, measures = None, drilldown = None, order = None, **options):
-        """See :meth:`cubes.browsers.Cuboid.aggregate`."""
+    def aggregate(self, cell, measures=None, drilldown=None, order=None, **options):
+        """See :meth:`cubes.browsers.cell.aggregate`."""
+
         result = cubes.browser.AggregationResult()
         
         # Create query
-        query = CubeQuery(cuboid, self.view, locale = self.locale)
+        query = CubeQuery(cell, self.view, locale=self.locale)
         query.drilldown = drilldown
         query.order = order
         query.prepare()
 
         ############################################
         # Get summary
-        cursor = self.connection.execute(query.summary_statement)
+        cursor = self.engine.execute(query.summary_statement)
         row = cursor.fetchone()
         summary = {}
         if row:
@@ -143,7 +156,7 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
             if page is not None and page_size is not None:
                 statement = statement.offset(page * page_size).limit(page_size)
             
-            rows = self.connection.execute(statement)
+            rows = self.engine.execute(statement)
             # print "SQL:\n%s"% statement
             # FIXME: change this into iterable, do not fetch everythig - we want to get plain dict
             # fields = query.fields + query.drilldown_fields
@@ -157,7 +170,7 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
                 records.append(record)
 
             count_statement = query.full_drilldown_statement.alias().count()
-            row_count = self.connection.execute(count_statement).fetchone()
+            row_count = self.engine.execute(count_statement).fetchone()
             total_cell_count = row_count[0]
 
             result.drilldown = records
@@ -165,10 +178,11 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
 
         return result
 
-    def facts(self, cuboid, order = None, **options):
+    def facts(self, cell, order = None, **options):
         """Retruns iterable objects with facts"""
         # Create query
-        query = CubeQuery(cuboid, self.view, locale = self.locale, **options)
+
+        query = CubeQuery(cell, self.view, locale = self.locale, **options)
         query.order = order
         query.prepare()
         statement = query.facts_statement
@@ -179,7 +193,7 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
         if page is not None and page_size is not None:
             statement = statement.offset(page * page_size).limit(page_size)
 
-        result = self.connection.execute(statement)
+        result = self.engine.execute(statement)
 
         return FactsIterator(result)
         
@@ -190,7 +204,7 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
 
         statement = query.fact_statement(key)
 
-        cursor = self.connection.execute(statement)
+        cursor = self.engine.execute(statement)
         
         row = cursor.fetchone()
         if row:
@@ -202,11 +216,11 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
             
         return record
         
-    def values(self, cuboid, dimension, depth = None, order = None, **options):
-        """Get values for dimension at given path within cuboid"""
+    def values(self, cell, dimension, depth = None, order = None, **options):
+        """Get values for dimension at given path within cell"""
 
         dimension = self.cube.dimension(dimension)
-        query = CubeQuery(cuboid, self.view, locale = self.locale, **options)
+        query = CubeQuery(cell, self.view, locale = self.locale, **options)
         query.order = order
 
         statement = query.values_statement(dimension, depth)
@@ -217,7 +231,7 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
         if page is not None and page_size is not None:
             statement = statement.offset(page * page_size).limit(page_size)
 
-        rows = self.connection.execute(statement)
+        rows = self.engine.execute(statement)
 
         fields = rows.keys()
         
@@ -233,13 +247,13 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
 CellAttribute = collections.namedtuple("CellAttribute", "attribute, name, column")
 
 class CubeQuery(object):
-    """docstring for CuboidQuery"""
-    def __init__(self, cuboid, view, locale = None, **options):
+    """docstring for CubeQuery"""
+    def __init__(self, cell, view, locale = None, **options):
         """Creates a cube query.
         
         :Attributes:
         
-            * `cuboid` - cuboid within query will be executed
+            * `cell` - cell within query will be executed
             * `view` - denormalized view/table where data is stored
             * `locale` - locale to be used for fetching data. if none specified, then default
               locale is used (first locale for attributes with multiple locales)
@@ -250,7 +264,7 @@ class CubeQuery(object):
         """
         
         super(CubeQuery, self).__init__()
-        self.cuboid = cuboid
+        self.cell = cell
 
         self.view = view
         self.condition_expression = None
@@ -263,7 +277,7 @@ class CubeQuery(object):
         
         self._prepared = False
 
-        self.cube = cuboid.cube
+        self.cube = cell.cube
         self.cube_key = self.cube.key
         if not self.cube_key:
             self.cube_key = base.DEFAULT_KEY_FIELD
@@ -520,15 +534,15 @@ class CubeQuery(object):
         self._conditions = []
         self._group_by = []
 
-        for cut in self.cuboid.cuts:
+        for cut in self.cell.cuts:
             dim = self.cube.dimension(cut.dimension)
             if isinstance(cut, cubes.browser.PointCut):
                 path = cut.path
-                condition = self._path_condition(dim, path)
+                condition = self._point_condition(dim, path)
             elif isinstance(cut, cubes.browser.SetCut):
                 conditions = []
                 for path in cut.paths:
-                    conditions.append(self._path_condition(dim, path))
+                    conditions.append(self._point_condition(dim, path))
                 condition = expression.or_(*conditions)
             else:
                 raise Exception("Only point and set cuts are supported in SQL browser at the moment")
@@ -536,7 +550,7 @@ class CubeQuery(object):
         
         self._condition = expression.and_(*self._conditions)
         
-    def _path_condition(self, dim, path):
+    def _point_condition(self, dim, path):
         """Adds a condition for `dimension` point at `path`."""
         conditions = [] 
         levels = dim.default_hierarchy.levels
@@ -606,55 +620,116 @@ class CubeQuery(object):
         considered"""
 
         if type(self.drilldown) == list or type(self.drilldown) == tuple:
-            self.logger.debug("normalizing drill-down")
+            self.logger.debug("converting drill-down specification to a dictionary")
             self._drilldown = {}
 
-            for obj in self.drilldown:
-                dim = self.cube.dimension(obj)
-                last_level = self._last_levels.get(dim.name)
-
-                if last_level:
-                    next_level = dim.default_hierarchy.next_level(self._last_levels[dim.name])
-                    if not next_level:
-                        raise ValueError("Unable to drill-down after level '%s'. It is last level "
-                                         "in default hierarchy in dimension '%s'" % 
-                                         (last_level.name, dim.name))
-                else:
-                    next_level = dim.default_hierarchy.levels[0]
-
-                self.logger.debug("dimension %s last level: %s next: %s" % (dim.name, last_level, next_level.name))
+            for dim in self.drilldown:
+                dim = self.cube.dimension(dim)
+                next_level = self._next_drilldown_level(dim)
                 self._drilldown[dim.name] = next_level
         elif isinstance(self.drilldown, dict):
-            self.logger.debug("no normalization of drill-down required")
-            self._drilldown = self.drilldown
+            self.logger.debug("updating next levels in drill-down dictionary")
+
+            # FIXME: shouldn't it be an ordered dictionary?
+            self._drilldown = {}
+            
+            for dim, level in self.drilldown.items():
+                dim = self.cube.dimension(dim)
+                if level:
+                    self._drilldown[dim.name] = level
+                else:
+                    next_level = self._next_drilldown_level(dim)
+                    self._drilldown[dim.name] = next_level
         else:
             raise TypeError("Drilldown is of unknown type: %s" % self.drilldown.__class__)
 
-    # def attribute(self, field):
-    #     """Return Attribute object based on field specification."""
-    #     if isinstance(field, cubes.model.Attribute):
-    #         return field
-    #         
-    #     split = field.split(".")
-    #     if len(split) == 1:
+    def _next_drilldown_level(self, dimension):
+        """Get next drilldown level for dimension. If we are already cutting the dimension, then return
+        next level to the last cut level. If we are not cutting, return first level."""
+        
+        # FIXME: only default hierarchy is currently used
+        
+        dim = self.cube.dimension(dimension)
+        last_level = self._last_levels.get(dim.name)
+        if last_level:
+            next_level = dim.default_hierarchy.next_level(self._last_levels[dim.name])
+            if not next_level:
+                raise ValueError("Unable to drill-down after level '%s'. It is last level "
+                                 "in default hierarchy in dimension '%s'" % 
+                                 (last_level.name, dim.name))
+        else:
+            next_level = dim.default_hierarchy.levels[0]
+
+        return next_level
         
     def column(self, field, dimension = None):
+        """Return a table column for `field` which can be either :class:`cubes.Attribute` or a string.
+        
+        Possible column names:
+        * ``field`` for fact field or flat dimension
+        * ``field.locale`` for localized fact field or flat dimension
+        * ``dimension.field`` for multi-level dimension field
+        * ``dimension.field.locale`` for localized multi-level dimension field
+        """
+
         # FIXME: should use: field.full_name(dimension, self.locale)
         # if there is no localization for field, use default name/first locale
         locale_suffix = ""
 
+        if isinstance(field, cubes.model.Attribute) and field.locales:
+            locale = self.locale if self.locale in field.locales else field.locales[0]
+            locale_suffix = "." + locale
+
         if dimension:
-            # FIXME: this will not work when fact decoration/detail attributes will be introduced
-            if isinstance(field, cubes.model.Attribute) and field.locales:
-                if self.locale in field.locales:
-                    locale = self.locale
-                else:
-                    locale = field.locales[0]
-                locale_suffix = "." + locale
-            logical_name = dimension.name + '.' + str(field)
+            # FIXME: temporary flat dimension hack, not sure about impact of this to other parts of the
+            # framework
+            if not dimension.is_flat or dimension.has_details:
+                logical_name = dimension.name + '.' + str(field)
+            else:
+                logical_name = str(field)
         else:
             logical_name = field
 
+        self.logger.debug("getting column %s(%s) loc: %s - %s" % (field, type(field), self.locale, locale_suffix))
+
         localized_name = logical_name + locale_suffix
         column = self.view.c[localized_name]
         return expression.label(logical_name, column)
+
+class SQLWorkspace(object):
+    """Factory for browsers"""
+    def __init__(self, model, engine, schema = None, name_prefix = None, name_suffix = None):
+        """Create a workspace"""
+        super(SQLWorkspace, self).__init__()
+        self.model = model
+        self.engine = engine
+        self.metadata = sqlalchemy.MetaData(bind = self.engine)
+        self.name_prefix = name_prefix
+        self.name_suffix = name_suffix
+        self.views = {}
+        self.schema = schema
+        
+    def browser_for_cube(self, cube, locale = None):
+        """Creates, configures and returns a browser for a cube"""
+        cube = self.model.cube(cube)
+        view = self.view_for_cube(cube)
+        browser = SQLBrowser(cube, view = view, locale = locale)
+        return browser
+        
+    def view_for_cube(self, cube, view_name = None):
+        if cube.name in self.views:
+            view = self.views[cube.name]
+        else:
+            if not view_name:
+                if self.name_prefix:
+                    view_name = self.name_prefix
+                else:
+                    view_name = ""
+                view_name += cube.name
+                if self.name_suffix:
+                    view_name += self.name_suffix
+
+            view = sqlalchemy.Table(view_name, self.metadata, autoload = True, schema = self.schema)
+            self.views[cube.name] = view            
+            
+        return view
